module PepaTypes (
		  Identifier, Rate, PEPArate,
		  isRateSymbol, processPEPArate, fromPEPArate, mkLbl
		 ) where

import Utils ((?),fromJust,isNothing)
import Maybe (listToMaybe)


type Identifier = String
data Rate = Symbol Identifier | Numeric Double deriving Eq
data PEPArate = PEPArt Rate | PEPAtop deriving Show

isRateSymbol :: Rate -> Bool
isRateSymbol (Symbol _ ) = True
isRateSymbol _ = False

instance Show Rate where
    show (Symbol id) = id
    show (Numeric n) = show n

instance Num Rate where
    (+) (Symbol s) (Numeric r)   = (Symbol s) + (Symbol $ show r)
    (+) (Numeric r) (Symbol s)   = (Symbol s) + (Numeric r)
    (+) (Numeric r) (Numeric r') = Numeric $ r+r'
    (+) (Symbol s) (Symbol t)    = Symbol $ concat ["(",s, ") + (", t, ")"]
    (*) _ _ = error "Num.Rate: (*) not needed"
    signum _ = error "Num.Rate: signum not needed"
    abs _ = error "Num.Rate: abs not needed"
    fromInteger _ = error "Num.Rate: fromInteger not needed"

instance Ord Rate where
    min (Symbol s) (Numeric r) =  min (Symbol s) (Symbol $ show r)
    min (Numeric r) (Symbol s) =  min (Symbol s) (Numeric r)
    min (Numeric r) (Numeric r') = Numeric $ min r r'
    min (Symbol s) (Symbol t)
	| s == t    = (Symbol s)
	| otherwise = (Symbol $ concat ["ipcmin((",s,"), (",t,"))"] )


-- Interprets the rate identifier to determine whether it is a real value, a symbolic variable or Top
processPEPArate :: Identifier -> PEPArate
processPEPArate lb
    | lb == "infty"       = PEPAtop
    | isNothing res       = PEPArt $ Symbol ("PEPA_"++lb)
    | (null rst && i > 0) = PEPArt $ Numeric i
    | otherwise           = error "processRate: badly formed numeric rate" where
			    (i, rst) = fromJust res
			    res = listToMaybe $ readsPrec 0 lb

-- List of rates stored so that the min function can eliminate duplicate min'd symbols if necessary
fromPEPArate :: PEPArate -> Maybe [Rate]
fromPEPArate PEPAtop = Nothing
fromPEPArate (PEPArt r) = Just [r]

-- Creates indexed PEPA component/DNAmaca transition labels
mkLbl :: (Identifier, Int) -> Identifier
mkLbl (lb,n) = (n < 0) ? (error "mkLbl: negative argument", concat [lb,"_",show n])
-- mkLbl (lb,n) = (n==0) ? (lb, concat [lb,"_",show n])

