module Utils (
	selectTerm, replaceTerm, isNothing, nSpace, nS, (?), (==>),
	showCol, showTable, ColWidth, ColJustify(..), choose, factorial,
	isJust,fromJust,isNothing,fromLeft,fromRight,isLeft,isRight,
	findInPair, findInPairErr, getPartitions, indent, numberLines
	) where

-- Utils v0.755 (HS98); (c)030423; jtb.

import Maybe (isJust,fromJust,isNothing)
import List (find,(\\),transpose)

-- <GHCi compatability mode>
fromInt :: (Num b, Integral a) => a -> b
fromInt = fromIntegral
-- </GHCi compatability mode>

numberLines :: [String] -> String
numberLines ss = showTableVarCol [[show i, ":", s] | (i, s) <- zip [1..] ss]

-- selects the first item from a list (if it exists) satisfying f and presents the item and the list\item
selectTerm :: (a -> Bool) -> [a] -> Maybe (a, [a])
selectTerm f ys
	| null ys2	= Nothing   
	| otherwise	= Just (head ys2, ys1 ++ tail ys2) where
                                (ys1, ys2) = span (not.f) ys

-- replaces elems in list xs that pass f with y
replaceTerm :: (a -> Bool) -> a -> [a] -> [a]
replaceTerm f y xs = map (\x -> f x ? (y,x)) xs 

-- finds b in [(a,b)] by using index element a
findInPair :: Eq a => a -> [(a,b)] -> Maybe b
findInPair z ys = isNothing mfd ? (Nothing, Just $ snd $ fromJust mfd) where
		  mfd = find (\x -> fst x == z) ys
-- as above but produces an error if not found
findInPairErr :: Eq a => (String,String) -> a -> [(a,b)] -> b
findInPairErr (fn,mess) x ys
    | isNothing mfd = error (fn ++ ": " ++ mess)
    | otherwise = fromJust mfd where
		  mfd = findInPair x ys


isLeft, isRight :: Either a b -> Bool
isLeft (Left _) = True
isLeft _ = False
isRight = not.isLeft
fromLeft :: Either a b -> a
fromLeft (Left x) = x
fromRight :: Either a b -> b
fromRight (Right y) = y

(?) :: Bool -> (a, a) -> a
(?) bl e = if bl then (fst e) else (snd e)

(==>) :: Bool -> Bool -> Bool
(==>) a b = (not a) || b

-- String Related


nS = nSpace			--alias
nSpace :: Int -> String
nSpace n
	| n < 0		= "" -- error "Utils.nSpace: negative argument"
	| otherwise	= concat $ take n $ repeat " "


-- Maths funclets
{-
permutations :: [a] -> [[a]]
permutations xs = map (\(y,ys) -> y:ys) $ getPartitions xs
-}
getPartitions :: [a] -> [(a, [a])]
getPartitions ls = partitions ls [] where
	partitions :: [a] -> [a] -> [(a, [a])]
	partitions [] _ = []
	partitions (x:xs) ys = (x, ys ++ xs) : partitions xs (x:ys)

factorial :: Int -> Int
factorial  0 = 1
factorial  1 = 1
factorial  2 = 2
factorial  3 = 6
factorial  4 = 24
factorial  5 = 120
factorial  6 = 720
factorial  7 = 5040
factorial  8 = 40320
factorial  9 = 362880
factorial 10 = 3628800
factorial 11 = 39916800
factorial 12 = 479001600
factorial n  = error ("Utils.factorial: Int overflow for " ++ (show n) ++ "!")

divFactorial :: Integer -> Int -> Integer
divFactorial p q 
	| q < 13	= div p (fromInt $ factorial q)
	| otherwise	= foldl div p $ map fromInt [1..q]

divFactDouble :: Double -> Int -> Double
divFactDouble p q
	| q < 13	= p / (fromIntegral $ factorial q)
	| otherwise	= foldl (/) p $ map fromInt [1..q]

choose  :: Int -> Int -> Integer
choose n r
	| r > n		= error "Utils.choose: r > n in nCr"
	| n == r	= 1
	| n < 13	= fromInt $ div (div (factorial n) (factorial $ n - r)) (factorial r)
	| otherwise	= (2*r < n) ? (f n r, f n (n-r)) where
		f = \x -> \y -> divFactorial (product $ g [x - y + 1..x]) y
		g = \xs -> map fromInt xs

indent :: String -> String
indent inset = unlines $ map ('\t':) (lines inset)

-- Column display routines

data ColJustify = L | C | R
type ColWidth = Int

-- If string s exceeds column width then it is allowed to overflow
showCol :: ColWidth -> ColJustify -> String -> String
showCol w j s
	| length s > w		= case j of
		R	-> (w > 0) ? ("$" ++ (drop (1 + (length s) - w) s), "")
		_	-> (w > 0) ? ((take (w - 1) s) ++ "$", "")
	| otherwise		= case j of
		L	-> s ++ (nS $ w - length s)
		C	-> sp1 ++ s ++ sp2 where
				sp = w - length s
				sp1 = nS (div sp 2)
				sp2 = (mod sp 2 == 1) ? (sp1 ++ " ", sp1)  
		R	-> (nS $ w - length s) ++ s


showTableVarCol :: [[String]] -> String
showTableVarCol xss = showTable cs xss where
	ns = map maximum $ map (map length) (transpose xss)
	cs = [((1,""),n,L) | n <- ns]

-- 1st cw param indicates separation between cols, 2nd is width of column itself
showTable :: [((ColWidth, String), ColWidth, ColJustify)] -> [[String]] -> String
showTable _ [] = ""
showTable forms (xs:xss) = (showRow $ zip forms xs) ++ (showTable forms xss) where
	showRow :: [(((ColWidth, String), ColWidth, ColJustify), String)] -> String
	showRow [] = "\n"
	showRow ((((ws, cdv), wc, just), str):cs) = (showCol ws C cdv) ++ (showCol wc just str) ++ (showRow cs)


